package org.pathwaycommons.trans;

/**
 * BIND elements/atts tags, as well as controlled vocab.
 * 
 * This class mostly lists the targetted BIND elements that carry 'real data'
 * in them, in addition to all of their parents that are needed to identify a
 * unique path to the target element in the XML document. Its usage is coupled
 * with the XML stats output document.
 * BIND complexes also use the shared tags: PubMedId, Pub_muid, Interaction_id,
 * Domain-id, Geninfo-id, BIND-object-type-id_protein, BIND-object-type-id_small-molecule,
 * BIND-object-type-id_dna, BIND-object-type-id_rna, BIND-object-type-id_complex,
 * Org-ref_taxname, BIND-other-db_dbname, BIND-other-db_intp, BIND-other-db_strp,
 * BIND-Rec-coll-descr_descr.
 *
 * checkBINDDiv () method, and its call from the BIND Trans is needed to clear
 * bad data.
 * 
 * @author rashadbadrawi
 */

public final class BINDTags {

    //BIND elements
    protected static final String INTERACTION = "BIND-Interaction";
    protected static final String INTERACTION_COL = "BIND-Rec-coll-descr_descr";
    protected static final String INTERACTION_SOURCE = "BIND-Interaction_source";
    protected static final String SIMPLE_DESC = "BIND-descr_simple-descr";
    protected static final String INTERACTION_ID = "Interaction-id";
    protected static final String PUB_OBJ = "BIND-pub-object";
    protected static final String PUBMED_ID = "PubMedId";
    protected static final String PUBMEDMU_ID = "Pub_muid";
    protected static final String EXP_SYS = "BIND-experimental-system";
    protected static final String COND = "BIND-condition";
    protected static final String COND_DESC = "BIND-condition_descr";
    protected static final String INTERACT_A = "BIND-Interaction_a";
    protected static final String INTERACT_B = "BIND-Interaction_b";

    protected static final String BIND_ID = "BIND-id";
    protected static final String BIND_ID_DI = "BIND-id_di";
    protected static final String BIND_ID_GI = "BIND-id_gi";
    //different object types
    protected static final String OBJ_TYPE_ID_COM = "BIND-object-type-id_complex";
    protected static final String OBJ_TYPE_ID_DNA = "BIND-object-type-id_dna";
    protected static final String OBJ_TYPE_ID_GENE = "BIND-object-type-id_gene";
    protected static final String OBJ_TYPE_ID_NS = "BIND-object-type-id_not-specified";
    protected static final String OBJ_TYPE_ID_PH = "BIND-object-type-id_photon";
    protected static final String OBJ_TYPE_ID_P = "BIND-object-type-id_protein";
    protected static final String OBJ_TYPE_ID_RNA = "BIND-object-type-id_rna";
    protected static final String OBJ_TYPE_ID_SM = "BIND-object-type-id_small-molecule";

    protected static final String OBJ_SHORT_LABEL = "BIND-object_short-label";
    protected static final String OBJ_DESC = "BIND-object_descr";
    protected static final String OBJ_SYN = "BIND-object_other-names_E";
    protected static final String OBJ_EXTREF = "BIND-object_extref";
    protected static final String ID_GI = "Geninfo-id";
    protected static final String OTHERDB = "BIND-other-db";
    protected static final String OTHERDB_NAME = "BIND-other-db_dbname";
    protected static final String OTHERDB_STRP = "BIND-other-db_strp";
    protected static final String OTHERDB_INTP = "BIND-other-db_intp";
    protected static final String DOMAIN_ID = "Domain-id";
    protected static final String ORG_TAXNAME = "Org-ref_taxname";
    protected static final String OBJECT_ID_ID = "Object-id_id";
    protected static final String SMALL_MOLECULE_ID = "Internal-small-molecule-id";

    //BIND Elements - complexes
    protected static final String SUBMIT_COMPLEXES = "BIND-Submit_complexes";
    protected static final String COMPLEX = "BIND-Molecular-Complex";
    protected static final String COMPLEX_MOL_OBJ = "BIND-mol-object";
    protected static final String COMPLEX_MOLOBJ_ID = "BIND-mol-object-id";  //order
    protected static final String COMPLEX_MOL_OBJ_SRC_A = "BIND-mol-object-source_a";
    protected static final String COMPLEX_MOL_OBJ_SRC_B = "BIND-mol-object-source_b";
    protected static final String COMPLEX_DESC = "BIND-Molecular-Complex_descr";
    protected static final String COMPLEX_NUM = "BIND-mol-sub-num_num";
    protected static final String COMPLEX_ORDERED_FLAG = "BIND-Molecular-Complex_ordered";
    protected static final String COMPLEX_REF = "BIND-Molecular-Complex_source"; //for pubmeds
    protected static final String COMPLEX_INT_LIST = "BIND-Molecular-Complex_interaction-list"; //for interactionID(s)
    protected static final String COMPLEX_MCID = "BIND-Molecular-Complex_mcid"; //to get the complex ID
    protected static final String COMPLEX_ID = "Molecular-Complex-id";    //complex ID

    //BIND attributes
    protected static final String VALUE_ATT = "value";

    protected static final String PUB_ID = "12519993";
    protected static final String UNKNOWN = "Unknown";     //safer-ignore case
    protected static final String DIV_TAXROOT = "BIND Taxroot";
    protected static final String DIV_TAXROOT_BOGUS =
       "BIND Taxroot|Record where all molecules are from any organism except those within TaxID 4751 or TaxID 33208";

    //Special CV Mapping Case-WormBase representation in BIND
    protected static final String SC_WB = "WormBase";
    protected static final String SC_WB_PAT_1 = "WBGene";
    protected static final String SC_WB_PAT_2 = "WP";
    protected static final String SC_WB_PAT_3 = "CE";

    //these are specific for BIND data bugs.
    protected static final String  NULL_VAL = "NULL";
    protected static final String NULL_VAL2 = "0";
    protected static final String NULL_VAL3 = "NULL.";
    protected static final String NULL_VAL4 = "WP:NULL";
    protected static final String NULL_VAL5 = "unknown";
    protected static final String NULL_VAL6 = "unknown.";

    protected static final String SOURCE_FILE_PREFIX = "taxid";

    protected static String checkBINDDiv (String currentBINDDiv) {

        if (currentBINDDiv.contains (BINDTags.DIV_TAXROOT_BOGUS)) {
            return BINDTags.DIV_TAXROOT;
        }

        return currentBINDDiv;
    }

}